"use strict";
const clamp = (num, min, max) => Math.min(Math.max(num, min), max);
class Vector {
    x;
    y;
    z;
    constructor(x = 0, y = 0, z = 0) {
        if (x instanceof Vector) {
            this.x = x.x;
            this.y = x.y;
            this.z = x.z;
        }
        else {
            this.x = x;
            this.y = y;
            this.z = z;
        }
    }
    add(x = 0, y = 0, z = 0) {
        if (x instanceof Vector)
            return this.add(x.x, x.y, x.z);
        this.x += x;
        this.y += y;
        this.z += z;
        return this;
    }
    sub(x = 0, y = 0, z = 0) {
        if (x instanceof Vector)
            return this.sub(x.x, x.y, x.z);
        this.x -= x;
        this.y -= y;
        this.z -= z;
        return this;
    }
    scale(n) {
        this.x *= n;
        this.y *= n;
        this.z *= n;
        return this;
    }
    lenSqr() {
        const { x, y, z } = this;
        return x * x + y * y + z * z;
    }
    len() {
        return Math.sqrt(this.lenSqr());
    }
    normalize() {
        const len = this.len();
        len !== 0 && this.scale(1 / len);
        return this;
    }
    dist(x = 0, y = 0, z = 0) {
        if (x instanceof Vector)
            return x.copy().sub(this).len();
        return this.dist(new Vector(x, y, z));
    }
    equals(x = 0, y = 0, z = 0) {
        if (x instanceof Vector)
            return this.equals(x.x, x.y, x.z);
        return this.x === x && this.y === y && this.z === z;
    }
    dot(x = 0, y = 0, z = 0) {
        if (x instanceof Vector)
            return this.dot(x.x, x.y, x.z);
        return this.x * x + this.y * y + this.z * z;
    }
    cross(x = 0, y = 0, z = 0) {
        if (x instanceof Vector)
            return this.cross(x.x, x.y, x.z);
        return new Vector(this.y * z - this.z * y, this.z * x - this.x * z, this.x * y - this.y * x);
    }
    lerp(x = 0, y = 0, z = 0, percent = 0.5) {
        if (x instanceof Vector)
            return this.lerp(x.x, x.y, x.z, y || 0.5);
        this.x += percent * (x - this.x);
        this.y += percent * (y - this.y);
        this.z += percent * (z - this.z);
        return this;
    }
    slerp(x = 0, y = 0, z = 0, percent = 0.5) {
        if (!(x instanceof Vector))
            return this.slerp(new Vector(x, y, z), percent);
        let dot = this.dot(x);
        dot = clamp(dot, -1, 1);
        const theta = Math.acos(dot) * (y || 0.5);
        const relativeVec = x.copy().sub(this.copy().scale(dot));
        relativeVec.normalize();
        this.x = this.x * Math.cos(theta) + relativeVec.x * Math.sin(theta);
        this.y = this.y * Math.cos(theta) + relativeVec.y * Math.sin(theta);
        this.z = this.z * Math.cos(theta) + relativeVec.z * Math.sin(theta);
        return this;
    }
    nlerp(x = 0, y = 0, z = 0, percent = 0.5) {
        if (!(x instanceof Vector))
            return this.nlerp(new Vector(x, y, z), percent);
        return this.lerp(x, y || 0.5).normalize();
    }
    copy() {
        return new Vector(this);
    }
}
module.exports = Vector;
//# sourceMappingURL=index.js.map